# -*- encoding: utf-8 -*-
import os
import optparse
import sys

import data

def expand(path):
    return os.path.abspath(
        os.path.expanduser(
            os.path.expandvars(
                path)))

parser = optparse.OptionParser(
    usage="%prog -p PROJECT [-b BRANCH] -c SOURCE DEST",
    description="Ports the bazaar branch SOURCE to the git repository DEST "
                "under the branch name BRANCH. If the branch name is not "
                "provided, infer it from the SOURCE path"
)
parser.add_option(
    '-b', '--branch', help="Name under which the bazaar branch should be "
                           "imported in git. If not provided, uses the last "
                           "section of the SOURCE directory name")
parser.add_option(
    '-p', '--project', choices=['server', 'web', 'addons'],
    help="Original bazaar project, required, one of server, web or addons %choices")

parser.add_option(
    '-c', '--stdout', action='store_true', default=False,
    help="Output fast-import stream to stdout instead of importing it into DEST")

options, args = parser.parse_args()

if not options.project:
    raise parser.error("PROJECT is required")

if len(args) != 2:
    raise parser.error("SOURCE and DEST are required")

source, dest = map(expand, args)

if not os.path.isdir(os.path.join(source, '.bzr')):
    raise parser.error("SOURCE must be a bazaar repository")
if not os.path.isdir(os.path.join(dest, '.git')):
    raise parser.error("DEST must be a git repository")

if not options.branch:
    options.branch = filter(None, source.split(os.path.sep))[-1]

import bzr
import git

def counter(stream):
    i = 0
    for line in stream:
        if line.startswith('commit refs/heads/'):
            i += 1
            if i % 1000 == 0:
                print >>sys.stderr, '%d commits' % i
        yield line

def move_data(i, o):
    i = data.rewrite_coalesced(i)
    if options.project == 'addons':
        i = data.prefix_paths(i, 'addons')
    i = data.fuckyotags(i)
    o.writelines(counter(i))

with bzr.export(source, branch=options.branch) as input:
    if options.stdout:
        move_data(input, sys.stdout)
    else:
        with git.import_(dest) as output:
            move_data(input, output)
