from __future__ import print_function
import cPickle as pickle
import pkgutil
import re
import sys

mark_re = re.compile(
    r'^(from|merge) :\d+$',
)
def rewrite_coalesced(stream):
    """ Rewrites marks of a stream based on non-coalesced mark files, to
    correctly fit into a coalesced output repository
    """
    folded = pickle.loads(
        pkgutil.get_data(
            'data', 'coalesced.pickle'))

    for line in stream:
        # reused marks?
        if line.startswith('mark :'):
            folded.pop(line[len('mark '):].strip(), None)
        if not mark_re.match(line):
            yield line
            continue

        # ':' is included in coalesced pickle
        yield re.sub(
            r'(:\d+)$',
            lambda m: folded.get(m.group(1), m.group(1)),
            line)

def prefix_paths(stream, prefix):
    for line in stream:
        if line.startswith(('M ', 'D ', 'C ', 'R ')):
            # [ -~] is the range of ASCII printable characters. Not ideal as
            # it will not match localized paths, but just matching \S+ will
            # find "paths" within binary files...
            line = re.sub(
                r'^(M \d+ inline) ([ -~]+)$',
                r'\1 {}/\2'.format(prefix),
                line)
            line = re.sub(
                r'^D ([ -~]+)$',
                r'D {}/\1'.format(prefix),
                line)
            line = re.sub(
                r'^(C|R) ([ -~]+) ([ -~]+)$',
                r'\1 {0}/\2 {0}/\3'.format(prefix),
                line)

        yield line

def fuckyotags(stream):
    for line in stream:
        if line.startswith('reset refs/tags'):
            # skip from()
            next(stream)
            # may be empty line
            nxt = next(stream)
            if not nxt.isspace():
                yield nxt
            continue
        yield line
